"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.typeMarkerField = exports.editorSupportField = exports.clearTypeMarkersEffect = exports.addTypeMarkerEffect = void 0;
var _state = require("@codemirror/state");
var _view = require("@codemirror/view");
var _editorSupport = require("@neo4j-cypher/editor-support");
/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [http://neo4j.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

const editorSupportField = _state.StateField.define({
  create() {
    return new _editorSupport.CypherEditorSupport();
  },
  update(editorSupport, tr) {
    return editorSupport;
  }
});
exports.editorSupportField = editorSupportField;
const addTypeMarkerEffect = _state.StateEffect.define();
exports.addTypeMarkerEffect = addTypeMarkerEffect;
const clearTypeMarkersEffect = _state.StateEffect.define();
exports.clearTypeMarkersEffect = clearTypeMarkersEffect;
const typeMarkerField = _state.StateField.define({
  create() {
    return _view.Decoration.none;
  },
  update(typeMarkers, tr) {
    typeMarkers = typeMarkers.map(tr.changes);
    for (let e of tr.effects) {
      if (e.is(clearTypeMarkersEffect)) {
        typeMarkers = typeMarkers.filter(() => false);
      } else if (e.is(addTypeMarkerEffect)) {
        if (e.value.from !== e.value.to) {
          typeMarkers = typeMarkers.update({
            add: [_view.Decoration.mark({
              class: "cm-p-" + e.value.type
            }).range(e.value.from, e.value.to)]
          });
        }
      }
    }
    return typeMarkers;
  },
  provide: f => _view.EditorView.decorations.from(f)
});
exports.typeMarkerField = typeMarkerField;